## **Customs commands** 

### **Json format** 
> Always contains "command" and "data" keys.
```
{  
  "command": "custom command", // mandatory, respect character case, Lower CamelCase
  "data": {} // optional
}
```

#### **Generate a report about tmgmt** 
> A json file will be generated. use jsoneditoronline.org to view it.
```
{  
  "command": "report"
}
```

#### **Reset all settings** 
> You will have to restart the checklist process.
```
{  
  "command": "resetSettings"
}  
```

#### **Show Drupal proxy settings** 
> Sent to Drupal Logger.
```
{
  "command": "proxySettings",
  "data": {
    "mode": "logger"
  }
}
```
> Using Drupal Messenger (default).
```
{
  "command": "proxySettings",
  "data": {
    "mode": "messenger"
  }
}
```

#### **Change job state** 
```
useful states: 
0: Unprocessed (A new translation job.)
1: Active (A translation job that has been submitted to the translator.)
5: Finished (A job is marked as 'finished' after every single attached job item has been reviewed, accepted and saved.)
```   
```
{  
  "command": "jobState",  
  "data": {
    "job_id": 1,
    "new_state_id": 1
  }  
}  
```

#### **Change job item state** 
```
useful states: 
1: In progress (The translation job item is active and waiting to be translated.)
2: Needs review (The translation job item needs to be reviewed.)
3: Accepted (The translation job item has been reviewed and accepted. Drupal content is translated.)
```   
```
{  
  "command": "jobItemState",  
  "data": {
    "job_item_id": 1,
    "new_state_id": 1
  }  
}  
```
#### **Change multiple job item state** 
> With job items list.
```
{  
  "command": "multiJobItemState",  
  "data": {
    "job_item_ids": [1, 2, 3],
    "new_state_id": 1
  }  
}  
```
> All job items from a job.
```
{  
  "command": "multiJobItemState",  
  "data": {
    "job_id": 1,
    "new_state_id": 1
  }  
}  
```

#### **Generate a job report** 
> A json file will be generated. use jsoneditoronline.org to view it.  
> `include_job_items` and `include_messages` are optional.  
```
{  
  "command": "jobReport",  
  "data": {
    "job_id": 1,
    "include_job_items": true,
    "include_messages": true
  }  
}  
```
```
{  
  "command": "jobReport",  
  "data": {
    "job_id": 1,
  }  
}  
```

#### **Start multiple commands** 
> Works with all commands.
```
{
  "command": "multiCommand",
  "data": [
    {
      "command": "jobReport",
      "data": {
        "job_id": 1,
        "include_job_items": true
      }
    },
    {
      "command": "report"
    }
  ]
}
```

#### **update job fields** 
```
{  
  "command": "jobUpdate",  
  "data": {
    "job_id": 1,
    "fields": {
      "": "",
      "": ""
    }
  }  
}  
```

#### **update job item fields** 
```
{  
  "command": "jobItemUpdate",  
  "data": {
    "job_item_id": 1,
    "fields": {
      "": "",
      "": ""
    }
  }  
}  
```

#### **update multiple job item fields** 
> With job items list.
```
{  
  "command": "multiJobItemUpdate",  
  "data": {
    "job_item_ids": [1, 2, 3],
    "fields": {
      "field_1": "",
      "field_2": ""
    }
  }  
}  
```
> All job items from a job id.
```
{  
  "command": "multiJobItemUpdate",  
  "data": {
    "job_id": 1,
    "fields": {
      "field_1": "",
      "field_2": ""
    }
  }  
}
```
 
#### **clean tmgmt_cdt settings** 
> Remove old unused settings.
```
{  
  "command": "cleanSetup"
}  
```

#### **setup chosen module** 
> Install and setup the correct settings for chosen.
```
{  
  "command": "setupChosen"
}  
```

#### **update version values** 
> Store version.json from current, release and dev to state 'tmgmt_cdt.version'.
```
{  
  "command": "updateVersion"
}  
```

#### **update tmgmt_cdt configuration Provider** 
> Update `tmgmt.translator.tmgmt_cdt` Provider 
```
{
  "command": "updateProvider",
  "data": {
    "key1": {
      "sub1": 100,
      "sub2": "text"
    },
    "key2": 200
  }
}
```
