<?php

namespace Drupal\tmgmt_cdt\Entity;

use Drupal\Component\Serialization\Json;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\tmgmt\Entity\JobItem;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt_cdt\Form\TmgmtCdtCartItemForm;
use Drupal\tmgmt_cdt\TmgmtCdtFormatHelper;
use Drupal\tmgmt_cdt\TmgmtCdtHelper;

/**
 * Override classe JobItem.
 */
class TmgmtCdtJobItem extends JobItem
{

    use StringTranslationTrait;

    /**
     * For job items in cart without job attached.
     *
     * @var bool
     */
    public $onCart = false;

    /**
     * {@inheritdoc}
     */
    public function isAbortable()
    {
        return false;
    }

    /**
     * {@inheritdoc}
     */
    public function abortTranslation()
    {
        throw new TMGMTException('Function disabled. Please check the CdT documentation');
    }

    /**
     * {@inheritdoc}
     */
    public function acceptTranslation()
    {
        try {
            // To not break the tmgmt logic (source and target language at job level)
            $job = $this->getJob();
            $job->set('target_language', $this->get('cdt_target_language')->value);
            $job->save();

            $parent = parent::acceptTranslation();

            if (tmgmt_job_check_finished($job->id())) {
                $job->finished();
            }

            // And back to multiple.
            $job->set('target_language', 'multiple');
            $job->save();

            return $parent;

        } catch (\Exception $e) {

            \Drupal::messenger()->deleteAll();
            \Drupal::messenger()->addError(
                $this->t(
                    'Error accepting translation of Job jobItem @tjid: @error',
                    [' @tjid' => $this->id(), '@error' => $e->getMessage()]
                )
            );

        }

    }

    /**
     * {@inheritdoc}
     */
    protected function count(&$item)
    {
        @parent::count($item);
        if (!empty($item['#text'])) {
            if (\Drupal::service('tmgmt.data')->filterData($item)) {
                $this->setCdtSetting('links', TmgmtCdtFormatHelper::extractHrefs($item['#text']));
            }
        }
    }

    /**
     * Get cdt setting.
     */
    public function getCdtSetting($name, $default = false)
    {
        $cdt_settings = (array) @Json::decode($this->get('cdt_settings')->value);
        if (isset($cdt_settings[$name])) {
            return $cdt_settings[$name];
        } else {
            return $default;
        }
    }

    /**
     * Set cdt setting.
     */
    public function setCdtSetting($name, $data)
    {
        $cdt_settings = (array) @Json::decode($this->get('cdt_settings')->value);
        $cdt_settings[$name] = $data;
        $this->set('cdt_settings', Json::encode($cdt_settings));
    }
 
    /**
     * Calculate characters count including excluded fields.
     */
    public function translatableCharacterCount()
    {   
        $translate_settings = $this->getCdtSetting('translate_settings', array());
        $content_type = $this->get('item_type')->value;
        $this->onCart = true;
        $data = \Drupal::service('tmgmt.data')->filterTranslatable($this->getSourceData());

        $cdt_character_count = 0;
        foreach ($data as $key => $field) {

            // full mode by default : we use only exclusion list
            if ($translate_settings['translation_type'] === TmgmtCdtFormatHelper::DEFAULT || empty($translate_settings)) {

                if (TmgmtCdtHelper::isFieldExcluded($content_type . '][' . $key)) {
                    continue;
                }
            } else {
                // partial mode : we use only user selection from this item
                $field_name = str_replace('][', '|', $key);
                if ((bool) $translate_settings['fields'][$field_name]['translate'] === false) {
                    continue;
                }
            }

            $cdt_character_count += TmgmtCdtFormatHelper::characterCount($field['#text']);
        }

        $this->set('cdt_character_count', $cdt_character_count);
        $this->onCart = false;
        $this->save();

        return $cdt_character_count;
    }

    /**
     * {@inheritdoc}
     */
    public function recalculateStatistics()
    {

        if (!$this->isAccepted()) {
            //$this->set('cdt_character_count', 0);
        }

        parent::recalculateStatistics();

    }

    /**
     * {@inheritdoc}
     */
    public function getJob()
    {
        $job = parent::getJob();
        if (!$job && $this->onCart) {
            return new FakeTmgmtCdtJob();
        }
        if ($job) {
            $job->set('target_language', $this->get('cdt_target_language')->value);
        }

        return $job;
    }

}
