<?php

namespace Drupal\tmgmt_cdt\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\tmgmt\Form\CartForm;
use Drupal\tmgmt_cdt\Entity\TmgmtCdtJobItem;
use Drupal\tmgmt_cdt\Form\TmgmtCdtCartItemForm;
use Drupal\tmgmt_cdt\TmgmtCdtHelper;
use Drupal\tmgmt_cdt\TmgmtCdtFormatHelper;

/**
 * Source overview form.
 */
class TmgmtCdtCartForm extends CartForm
{

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state, $plugin = null, $item_type = null)
    {
        $languages = tmgmt_available_languages();
        $options = array();
        $selected = array();
        foreach (tmgmt_cart_get()->getJobItemsFromCart() as $item) {

            $source_plugin = $item->getSourcePlugin();
            //$is_editable = $item->get('item_type')->value == 'node' || $item->get('item_type')->value == 'webform' || $item->get('item_type')->value == 'view';
            $item->onCart = true;
            $data = \Drupal::service('tmgmt.data')->filterTranslatable($item->getSourceData());
            $is_editable = !empty($data);

            $translation_type = $item->getCdtSetting('translate_settings', array());

            $url_edit = Url::fromRoute('tmgmt_cdt.cart_item', ['id' => $item->id()]);

            $selected[$item->id()] = true;
            $options[$item->id()] = array(
                $item->getSourceType(),
                $item->label(),
                isset($languages[$item->getSourceLangCode()]) ? $languages[$item->getSourceLangCode()] : t('Unknown'),
                @$translation_type['translation_type'] === TmgmtCdtFormatHelper::CUSTOM ? TmgmtCdtFormatHelper::CUSTOM : TmgmtCdtFormatHelper::DEFAULT,
                $is_editable ? Link::fromTextAndUrl('edit', $url_edit) : '',
            );

        }

        $form['items'] = array(
            '#type' => 'tableselect',
            '#header' => array(t('Type'), t('Content'), t('Source language'), 'Translation type', 'Action'),
            '#empty' => t('There are no items in your cart.'),
            '#options' => $options,
            '#default_value' => $selected,
        );

        $form['request_translation'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => t('Request translation'),
            '#submit' => array('::submitRequestTranslation'),
            '#validate' => array('::validateRequestTranslation'),
        );

        $form['remove_selected'] = array(
            '#type' => 'submit',
            '#button_type' => 'danger',
            '#value' => t('Remove selected item'),
            '#submit' => array('::submitRemoveSelected')
        );

        $form['empty_cart'] = array(
            '#type' => 'submit',
            '#button_type' => 'danger',
            '#value' => t('Empty cart'),
            '#submit' => array('::submitEmptyCart'),
        );

        return $form;

    }

    /**
     * Custom form submit callback for tmgmt_cart_cart_form().
     */
    public function submitRequestTranslation(array $form, FormStateInterface $form_state)
    {

        $jobs = array();
        $remove_job_item_ids = array();

        $job = tmgmt_job_create(\Drupal::languageManager()->getDefaultLanguage()->getId(), 'multiple', $this->currentUser()->id());
        $job_empty = true;

        /** @var \Drupal\tmgmt\JobItemInterface $job_item */
        foreach (TmgmtCdtJobItem::loadMultiple(array_filter($form_state->getValue('items'))) as $job_item) {
            try {

                $job->addItem($job_item->getPlugin(), $job_item->getItemType(), $job_item->getItemId(), array(
                    'cdt_settings' => $job_item->get('cdt_settings')->value,
                    'cdt_character_count' => $job_item->get('cdt_character_count')->value,
                ));
                $remove_job_item_ids[$job_item->id()] = $job_item->id();
                $job_empty = false;

            } catch (\Exception $e) {
                $this->messenger()->addStatus($e->getMessage(), 'error');
            }

            if (!$job_empty) {
                $jobs[] = $job;
            }
        }

        // Remove job items from the cart.
        if ($remove_job_item_ids) {
            tmgmt_cart_get()->removeJobItems($remove_job_item_ids);
            $storage = \Drupal::entityTypeManager()->getStorage('tmgmt_job_item');
            if ($delete_items = $storage->loadMultiple($remove_job_item_ids)) {
                $storage->delete($delete_items);
            }
        }

        // Start the checkout process if any jobs were created.
        if ($jobs) {
            \Drupal::service('tmgmt.job_checkout_manager')->checkoutAndRedirect($form_state, $jobs);
        } else {
            $this->messenger()->addError(t('From the selection you made it was not possible to create any translation job.'));
        }
    }

    /**
     * {@inheritdoc}
     */
    public function validateRequestTranslation(array &$form, FormStateInterface $form_state)
    {
        $items = array_filter($form_state->getValue('items'));
        $errors = TmgmtCdtHelper::checkCartItems($items);
        if (!empty($errors)) {
            foreach ($errors as $message) {
                $form_state->setErrorByName('request_translation', $message);
            }
        }
    }

}
