<?php

namespace Drupal\tmgmt_cdt\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Render\Element;
use Drupal\tmgmt_cdt\Entity\TmgmtCdtJobItem;
use Drupal\tmgmt_cdt\TmgmtCdtFormatHelper;
use Drupal\tmgmt_cdt\TmgmtCdtHelper;

/**
 * Source overview form.
 */
class TmgmtCdtCartItemForm extends FormBase
{
 
    use MessengerTrait;

    /**
     * {@inheritdoc}
     */

    public function buildForm(array $form, FormStateInterface $form_state, $id = null)
    {

        $job_item = TmgmtCdtJobItem::load($id);
        $form_state->set('job_item', $job_item);
        $translate_settings = $job_item->getCdtSetting('translate_settings', array());
        $form_state->set('translate_settings', $translate_settings);
        $form_state->set('content_type', $job_item->get('item_type')->value);

        $form['#title'] = $this->t('Job item: (@source_label) on cart', array('@source_label' => $job_item->getSourceLabel()));

        $form['info'] = array(
            '#type' => 'container',
            '#attributes' => array('class' => array('tmgmt-ui-job-info', 'clearfix')),
            '#weight' => 0,
        );

        $url = $job_item->getSourceUrl();
        $form['info']['source'] = array(
            '#type' => 'item',
            '#title' => t('Source'),
            '#markup' => $url ? Link::fromTextAndUrl($job_item->getSourceLabel(), $url)->toString() : $job_item->getSourceLabel(),
            '#prefix' => '<div class="tmgmt-ui-source tmgmt-ui-info-item">',
            '#suffix' => '</div>',
        );
        $form['info']['sourcetype'] = array(
            '#type' => 'item',
            '#title' => t('Source type'),
            '#markup' => $job_item->getSourceType(),
            '#prefix' => '<div class="tmgmt-ui-source-type tmgmt-ui-info-item">',
            '#suffix' => '</div>',
        );
        $form['info']['translation_type'] = array(
            '#type' => 'item',
            '#title' => t('Translation type'),
            '#markup' => @$translate_settings['translation_type'] === TmgmtCdtFormatHelper::CUSTOM ? TmgmtCdtFormatHelper::CUSTOM : TmgmtCdtFormatHelper::DEFAULT,
            '#prefix' => '<div class="tmgmt-ui-source-type tmgmt-ui-info-item">',
            '#suffix' => '</div>',
        );

        $job_item->onCart = true;
        $data = $job_item->getSourceData();

        foreach (Element::children($data) as $key) {
            $element = $this->selectableFormElement($form_state, \Drupal::service('tmgmt.data')->flatten($data[$key], $key));
            if ($element) {
                $form[$key] = $element;
            }
        }

        $form['actions'] = [
            '#type' => 'actions',
        ];

        $form['actions']['submit_save'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#weight' => 1,
            '#value' => $this->t('Save your changes'),
        );

        $form['actions']['submit_recommanded'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#weight' => 2,
            '#value' => $this->t('Save with recommanded translation'),
            '#submit' => ['::recommandedForm'],
        );

        $form['actions']['submit_reset'] = array(
            '#type' => 'submit',
            '#button_type' => 'danger',
            '#weight' => 2,
            '#value' => $this->t('Reset (remove translation changes)'),
            '#submit' => ['::resetForm'],
        );

        $form['actions']['submit_cancel'] = array(
            '#type' => 'submit',
            '#weight' => 3,
            '#value' => $this->t('Cancel'),
            '#submit' => ['::cancelForm'],
        );

        $form['#attached']['library'][] = 'tmgmt/admin';

        return $form;
    }

    /**
     * Build form elements for the review form using flattened data items.
     *
     * @todo Mention in the api documentation that the char '|' is not allowed in
     * field names.
     *
     * @param \Drupal\Core\Form\FormStateInterface $form_state
     *   The current state of the form.
     * @param array $data
     *   Flattened array of translation data items.
     *
     * @return array|NULL
     *   Render array with the form element, or NULL if the text is not set.
     */
    public function selectableFormElement(FormStateInterface &$form_state, $data)
    {
        $element = null;
        $translate_settings = (array) $form_state->get('translate_settings');

        foreach (Element::children($data) as $key) {
            $data_item = $data[$key];

            if (isset($data_item['#text']) && \Drupal::service('tmgmt.data')->filterData($data_item)) {
                // The char sequence '][' confuses the form API so we need to replace
                // it when using it for the form field name.
                $data_item['field_name'] = str_replace('][', '|', $key);
                $field_name = $data_item['field_name'];
                $full_filed_name = $form_state->get('content_type') . '][' . $key;

                if (!isset($translate_settings['fields'][$field_name])) {
                    $in_exclude_list = TmgmtCdtHelper::isFieldExcluded($full_filed_name);
                    // convert content and calculate characters
                    $content_to_translate = TmgmtCdtFormatHelper::characterCount($data_item['#text']) > 0;

                    $default_translate = $content_to_translate && !$in_exclude_list;
                    $translate_settings['fields'][$field_name] = array(
                        'in_exclude_list' => $in_exclude_list,
                        'content_to_translate' => $content_to_translate,
                        'default_translate' => $default_translate,
                        'translate' => $default_translate,
                    );
                }
                $form_state->set('translate_settings', $translate_settings);

                // Ensure that the review form structure is initialized.
                $element['#theme'] = 'tmgmt_data_items_form';
                $element['#top_label'] = array_shift($data_item['#parent_label']);
                $leave_label = array_pop($data_item['#parent_label']);

                // Data items are grouped based on their key hierarchy, calculate the
                // group key and ensure that the group is initialized.
                $group_name = substr($field_name, 0, strrpos($field_name, '|'));
                if (empty($group_name)) {
                    $group_name = '_none';
                }
                if (!isset($element[$group_name])) {
                    $element[$group_name] = [
                        '#group_label' => $data_item['#parent_label'],
                    ];
                }

                // Initialize the form element for the given data item and make it
                // available as $element.
                $element[$group_name][$field_name] = array(
                    '#tree' => true,
                );
                $item_element = &$element[$group_name][$field_name];

                //$item_element['label']['#markup'] = $leave_label;
                $data_item['label'] = $leave_label;

                // Manage the height of the textareas, depending on the length of the
                // description. The minimum number of rows is 3 and the maximum is 15.
                $rows = ceil(strlen($data_item['#text']) / 100) + 2;
                $rows = min($rows, 15);
                $rows = max($rows, 3);

                // Allow other modules to change the source and translation texts,
                // for example to mask HTML-tags.
                $source_text = $data_item['#text'];

                // Build source and translation areas.

                $item_element = $this->buildSource($item_element, $source_text, $data_item, $rows, $form_state);

            }
        }

        return $element;
    }

    /**
     * Builds the source form elements for a data item.
     *
     * @param array $item_element
     *   The form element for the data item.
     * @param string $source_text
     *   The source's text to display in the item element.
     * @param array $data_item
     *   The data item.
     * @param int $rows
     *   The number of rows that should be displayed.
     * @param \Drupal\Core\Form\FormStateInterface $form_state
     *   The form state.
     *
     * @return array
     *   The form element for the data item.
     */
    protected function buildSource($item_element, $source_text, $data_item, $rows, FormStateInterface $form_state)
    {

        $field_name = $data_item['field_name'];
        $translate_settings = (array) $form_state->get('translate_settings');
        $content_converted = TmgmtCdtFormatHelper::convertDrupalToCdt($source_text) !== $source_text;

        $row_information = array(
            '#type' => 'table',
        );
        // $row_information[0] = array('Field reference', $form_state->get('content_type') . '][' . str_replace('|', '][', $field_name));

        $row_information[0] = array(
            'label' => array(
                '#type' => 'markup',
                '#markup' => $this->t('Field reference'),
            ),
            'value' => array(
                '#type' => 'markup',
                '#markup' => $form_state->get('content_type') . '][' . str_replace('|', '][', $field_name),
            ),
        );

        $row_information[3] = array();
        // $description = 'Field reference: <strong>' .  . '</strong>';

        if ((bool) $translate_settings['fields'][$field_name]['in_exclude_list']) {
            //$description .= '<br/>Present in the global exclusion list';
            //$row_information[1] = array('Present in the global exclusion list', 'Yes');
            $row_information[1] = array(
                'label' => array(
                    '#type' => 'markup',
                    '#markup' => $this->t('Present in the global exclusion list'),
                ),
                'value' => array(
                    '#type' => 'markup',
                    '#markup' => $this->t('Yes'),
                ),
            );
        }
        if (!(bool) $translate_settings['fields'][$field_name]['content_to_translate']) {
            //$description .= '<br/>No content to translate';
            // $row_information[2] = array('Content to translate', 'No');
            $row_information[2] = array(
                'label' => array(
                    '#type' => 'markup',
                    '#markup' => $this->t('Translatable content'),
                ),
                'value' => array(
                    '#type' => 'markup',
                    '#markup' => $this->t('No'),
                ),
            );
        }

        $row_information[4] = array(
            'label' => array(
                '#type' => 'markup',
                '#markup' => $this->t('Recommendation'),
            ),
            'value' => array(
                '#type' => 'markup',
                '#markup' => (bool) $translate_settings['fields'][$field_name]['default_translate'] ? $this->t('Translate') : $this->t('Do not translate'),
            ),
        );

        $field_info = array(
            '#type' => 'details',
            '#open' => false,
            '#title' => t('Details'),
            'table' => array(),
        );

        $item_element['source'] = array(
            'translate' => array(
                '#type' => 'checkbox',
                '#default_value' => (bool) $translate_settings['fields'][$field_name]['translate'],
                '#title' => '<strong>' . $this->t('Translate') . '</strong>',

            ),
            'label' => array(
                '#type' => 'label',
                '#title' => $data_item['label'],
            ),
            'field' => array(),
            'info' => $field_info,
        );

        $rows = ceil(strlen($data_item['#text']) / 100) + 2;
        $rows = min($rows, 15);
        $rows = max($rows, 3);

        if (!empty($data_item['#format'])) {

            $item_element['source']['field'] =
            array(
                '#type' => 'text_format',
                '#default_value' => $source_text,
                '#disabled' => true,
                '#rows' => $rows,
                '#allowed_formats' => array($data_item['#format']),
            );

        } else {

            $item_element['source']['field'] =
            array(
                '#type' => 'textarea',
                '#default_value' => $source_text,
                '#disabled' => true,
                '#rows' => $rows,
            );
        }

        if (TmgmtCdtFormatHelper::convertDrupalToCdt($source_text) !== $source_text) {
            $row_information[3] = array(
                'label' => array(
                    '#type' => 'markup',
                    '#markup' => $this->t('Converted content<br><i>The Drupal reserved keywords <br>will be protected to avoid their translation.<i>'),
                ),
                'disable_conversion' => array(),
                'value' => array(
                    '#type' => 'textarea',
                    '#default_value' => TmgmtCdtFormatHelper::convertDrupalToCdt($source_text),
                    '#disabled' => true,
                    '#rows' => $rows,
                ),
            );

            if (TmgmtCdtHelper::getTranslatorSetting('conf_features.cart_field_convert')) {
                $row_information[3]['disable_conversion'] = array(
                    '#type' => 'checkbox',
                    '#default_value' => (bool) $translate_settings['fields'][$field_name]['disable_conversion'],
                    '#title' => '<strong>' . $this->t('Disable conversion') . '</strong>',
                    '#description' => '<i>Translators will see original content <br>including Drupal reserved keywords. (<strong>Not recommended</strong>)<i>'
                );
            }
            
        }

        $item_element['source']['info']['table'] = $row_information;

        return $item_element;
    }

    /**
     * {@inheritdoc}
     */
    public function getFormId()
    {
        return 'tmgmt_cdt_cart_item_edit';
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {

       
        $translate_settings = (array) $form_state->get('translate_settings');

        $job_item = $form_state->get('job_item');

        // https://www.drupal.org/node/2310411

        $translate_default = true;
        foreach (array_keys($translate_settings['fields']) as $field) {
            $check = $form_state->getValue($field);

            if (isset($check)) {
               

                if (@$check['source']['translate']) {
                    $translate_settings['fields'][$field]['translate'] = true;
                    $disable_conversion = (bool) @$check['source']['info']['table'][3]['disable_conversion'];
                    $translate_settings['fields'][$field]['disable_conversion'] = $disable_conversion;
                  
                    if ($disable_conversion) {
                        $translate_default = false;
                    }
                } else {
                    $translate_settings['fields'][$field]['translate'] = false;
                    $translate_settings['fields'][$field]['disable_conversion'] = false;
                    $translate_default = false;
                }
            }
        }
        if ($translate_default) {
            $translate_settings['translation_type'] = TmgmtCdtFormatHelper::DEFAULT;
        } else {
            $translate_settings['translation_type'] = TmgmtCdtFormatHelper::CUSTOM;
        }

        $job_item->setCdtSetting('translate_settings', array(
            'translation_type' => $translate_settings['translation_type'],
            'fields' => $translate_settings['fields'])
        );

        $job_item->onCart = false;
        $job_item->save();

        $form_state->setRedirect('tmgmt.cart');

    }

    /**
     * {@inheritdoc}
     */
    public function cancelForm(array &$form, FormStateInterface $form_state)
    {
        // dpm('cancelForm', microtime());
        $form_state->setRedirect('tmgmt.cart');
    }

    /**
     * {@inheritdoc}
     */
    public function recommandedForm(array &$form, FormStateInterface $form_state)
    {
        //  dpm('recommandedForm', microtime());

        $translate_settings = (array) $form_state->get('translate_settings');
        $job_item = $form_state->get('job_item');

        $translate_default = true;
        foreach (array_keys($translate_settings['fields']) as $field) {
            $check = $form_state->getValue($field);
            if (isset($check)) {
                $translate_settings['fields'][$field]['translate'] = $translate_settings['fields'][$field]['default_translate'];
                $translate_settings['fields'][$field]['disable_conversion'] = false;
                if (!$translate_settings['fields'][$field]['default_translate']) {
                    $translate_default = false;
                }
            }
        }
        if ($translate_default) {
            $translate_settings['translation_type'] = TmgmtCdtCartItemForm::DEFAULT;
        } else {
            $translate_settings['translation_type'] = TmgmtCdtCartItemForm::CUSTOM;
        }

        $job_item->setCdtSetting('translate_settings', array(
            'translation_type' => $translate_settings['translation_type'],
            'fields' => $translate_settings['fields'])
        );

        $job_item->onCart = false;
        $job_item->save();

        $form_state->setRedirect('tmgmt.cart');

    }

    /**
     * {@inheritdoc}
     */
    public function resetForm(array &$form, FormStateInterface $form_state)
    {
        //dpm('resetForm', microtime());

        $job_item = $form_state->get('job_item');
        $job_item->setCdtSetting('translate_settings', array());
        $job_item->onCart = false;
        $job_item->save();

        $form_state->setRedirect('tmgmt.cart');

    }

}
