<?php

namespace Drupal\tmgmt_cdt\Plugin\tmgmt_cdt\Format;

use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\JobItemInterface;
use Drupal\tmgmt_cdt\TmgmtCdtFormatHelper;
use Drupal\tmgmt_cdt\TmgmtCdtHelper;
use Drupal\tmgmt_file\Format\FormatInterface;

/**
 * Manage the xml file.
 */
class TmgmtCdtXml extends \XMLWriter implements FormatInterface
{

    use MessengerTrait;
    use StringTranslationTrait;

    /**
     * Contains a reference to the currently being exported job.
     *
     * @var \Drupal\tmgmt\Entity\Job
     */
    protected $job;

    /**
     * Xml content.
     *
     * @var string
     */

    protected $importedXML;

    /**
     * Adds a job item to the xml export.
     */
    protected function addItem(JobItemInterface $item)
    {

        $this->startElement('Translation');

        $this->startElement('PreviewLink');
        $this->writeCdata('NO PREVIEW LINK');
        $this->endElement();

        $this->startElement('SourceReference');
        $this->writeCdata(TmgmtCdtFormatHelper::itemSourceReference($item)['link']);
        $this->endElement();

        $data = $data_ordered = \Drupal::service('tmgmt.data')->filterTranslatable($item->getData());

        // Find field(s) with "title" and put at the start.
        $content_type = $item->get('item_type')->value;
        if ($content_type !== 'view' && $content_type !== 'webform') {
            $titles = [];
            foreach ($data as $key => $field) {
                if (strpos(mb_strtolower($key), 'title') !== false) {
                    $titles[$key] = $data[$key];
                    unset($data[$key]);
                }
            }
            // Titles ..
            $data_ordered = [];
            foreach ($titles as $key => $field) {
                $data_ordered[$key] = $field;
            }
            // And other ...
            foreach ($data as $key => $field) {
                $data_ordered[$key] = $field;
            }
            unset($data);
        }

        $tags_type_count = $item->getCdtSetting('tagsTypeCount', array());

        $translate_settings = $item->getCdtSetting('translate_settings', array());

        if ($translate_settings['translation_type'] === TmgmtCdtFormatHelper::default || empty($translate_settings)) {
            $translation_type = TmgmtCdtFormatHelper::default;
            $translation_mode = TmgmtCdtHelper::getTranslatorSetting('conf_feature.fields_translation_mode');
            if ($translation_mode === TmgmtCdtHelper::AUTO) {
                // Detect translation auto // TODO
                $fields_translation_detected = TmgmtCdtFormatHelper::jobItemFieldsTranslationDetection($item);
            }
        } else {
            $translation_type = TmgmtCdtFormatHelper::CUSTOM;
        }

        $attributs_converted = array();

        foreach ($data_ordered as $key => $field) {

            $field_name = str_replace('][', '|', $key);

            // defaut mode
            if ($translation_type === TmgmtCdtFormatHelper::default) {

                switch ($translation_mode) {
                    case TmgmtCdtHelper::EXCLUSION_LIST:
                        if (TmgmtCdtHelper::isFieldExcluded($content_type . '][' . $key)) {
                            continue;
                        }
                        break;
                    case TmgmtCdtHelper::ALL:
                        // we exclude nothing
                        break;
                    case TmgmtCdtHelper::AUTO:
                    default:

                        if ($fields_translation_detected[$field_name]['in_exclude_list']) {
                            continue;
                        }
                        if (!$fields_translation_detected[$field_name]['content_to_translate']) {
                            continue;
                        }
                        break;
                }

            } else {
                // custom mode : we use only user selection from this item
                if ((bool) $translate_settings['fields'][$field_name]['translate'] === false) {
                    continue;
                }
            }

            $this->startElement('StaticContent');
            $this->writeAttribute('indexType', $item->get('item_type')->value);
            $this->writeAttribute('instanceId', $item->get('item_id')->value);
            $this->writeAttribute('resName', '][' . $key);

            if (TmgmtCdtFormatHelper::isHtml($field['#text'])) {
                $this->writeAttribute('resType', 'html');
            } else {
                $this->writeAttribute('resType', 'text');
            }

            if ((bool) @$translate_settings['fields'][$field_name]['disable_conversion'] === false) {
                $content = TmgmtCdtFormatHelper::convertDrupalToCdt($field['#text']);
                $attributs_converted = array_merge($attributs_converted, TmgmtCdtFormatHelper::convertedAttributs($content));
                $attributs_converted = array_unique($attributs_converted);
            } else {
                $content = $field['#text'];
            }
  
            if (isset($field['#max_length'])) {
                $this->writeAttribute('resMaxSize', (int) $field['#max_length']);
            } else {
                $this->writeAttribute('resMaxSize', '');
            }

            $this->writeAttribute('resLabel', @$field['#parent_label'][0] ? $field['#parent_label'][0] : 'Field');

            $_key = str_replace('[', '', $key);
            $_key = str_replace(']', '', $_key);

            $tags_type_count[$_key] = TmgmtCdtFormatHelper::tagsTypeCount($content);

            $this->writeCdata($content);
            unset($content);
            // End tag </StaticContent>.
            $this->endElement();
        }
        // End tag </Translation>.
        $this->endElement();

        $item->setCdtSetting('tagsTypeCount', $tags_type_count);
        $item->setCdtSetting('attributsConverted', $attributs_converted);
 
        $item->save();

    }

    /**
     * {@inheritdoc}
     */
    public function export(JobInterface $job, $conditions = [])
    {

        $this->job = $job;

        $this->openMemory();
        $this->setIndent(true);
        $this->setIndentString(' ');
        $this->startDocument('1.0', 'UTF-8', 'no');

        // <Transaction> Root element.
        $this->startElement('Transaction');
        $this->writeAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
        $this->writeAttribute('xsi:noNamespaceSchemaLocation', 'http://static.cdt.europa.eu/webtranslations/schemas/Drupal-Translation-V8-2.xsd');

        $this->startElement('TransactionHeader');
        $this->writeAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');

        $this->startElement('SenderDetails');
        $this->writeAttribute('drupalVersion', \DRUPAL::VERSION);
        $this->writeAttribute('tmgmtVersion', @\Drupal::service('extension.list.module')->getExtensionInfo('tmgmt')['version']);
        $this->writeAttribute('moduleVersion', @TmgmtCdtHelper::getState('version')['current']['version']);
        $this->writeElement('ProducerDateTime', date('c'));
        // End tag </SenderDetails>.
        $this->endElement();
        // End tag </TransactionHeader>.
        $this->endElement();

        $this->writeElement('TransactionIdentifier', $job->id());
        $this->writeElement('TransactionCode', 'Drupal Translation Request');

        $this->startElement('TransactionData');
        $this->startElement('TranslationDetails');

        // To be sure we export one item per language (the first one orignal lang)
        $items_unique = [];
        foreach ($job->getItems() as $item) {
            if (in_array($item->get('item_id')->value, $items_unique)) {
                continue;
            }
            $this->addItem($item);
            $items_unique[] = $item->get('item_id')->value;
        }
        unset($items_unique);

        $this->writeElement('TotalCharacterLength', $job->getCounts()['characters']);
        // End tag </TransactionDetails>.
        $this->endElement();
        // End tag </TransactionData>.
        $this->endElement();

        $this->endDocument();
        return $this->outputMemory();

    }

    /**
     * {@inheritdoc}
     */
    public function import($imported_file, $is_file = true)
    {
    }

    /**
     * {@inheritdoc}
     */
    public function importXml(int $job_id, string $xml_data, string $target_language)
    {
        try {
            if (!$this->importedXml) {
                throw new \Exception('Xml not valid for language ' . $target_language);
            }

            $transaction_identifier = $this->importedXml->xpath('//TransactionIdentifier');
            $transaction_identifier = (int) reset($transaction_identifier);

            if ($transaction_identifier !== $job_id) {
                throw new \Exception('Wrong TransactionIdentifier for language ' . $target_language);
            }

            $job = (Job::load((int) $transaction_identifier));

            $flat_data = $this->getImportedTargets($job, $target_language);
            // Free mem.
            $this->importedXml = null;
            $data = \Drupal::service('tmgmt.data')->unflatten((array) $flat_data);
            return $data;
        } catch (\Exception $e) {
            \Drupal::messenger()->addError($e->getMessage());
            return [];
        }

        if ($this->getImportedXML($imported_file, $is_file) === false) {
            return false;
        }

    }

    /**
     * {@inheritdoc}
     */
    public function validateImport($imported_file, $is_file = true)
    {
        try {

            $dom = new \DOMDocument();
            $dom->validateOnParse = true;

            @$dom->loadXML($imported_file);

            libxml_use_internal_errors(true);
            libxml_clear_errors();

            if (!(bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.disable_xsd_validation')) {

                $xml_xsd_link = trim($dom->documentElement->getAttributeNS($dom->lookupNamespaceURI('xsi'), 'noNamespaceSchemaLocation'));

                $schema_source = '';

                // Check if the current xsd is the same as the stored one.
                if (TmgmtCdtHelper::getTranslatorSetting('conf_advanced.xsd_link') !== $xml_xsd_link) {
                    $schema_source = TmgmtCdtHelper::curlGetFileContents($xml_xsd_link);
                    if ($schema_source !== false) {
                        \Drupal::state()->set('tmgmt_cdt.xsd_source', $schema_source);
                        \Drupal::state()->set('tmgmt_cdt.xsd_link', $xml_xsd_link);
                    } else {
                        throw new \Exception('Error reading content from ' . $xml_xsd_link);
                    }
                }

                $schema_source = TmgmtCdtHelper::getState('xsd_source');
                @$dom->schemaValidateSource($schema_source);

                $xml_errors = libxml_get_errors();
                if (\count($xml_errors) > 0) {
                    \Drupal::logger('tmgmt_cdt')->debug('CdT invalid XML<pre><code>' . var_export($xml_errors, true) . '</code></pre>');
                    throw new \Exception('XML could not be validated by the XSD schema. Compare the XML schema with the current configuration of the CdT module');
                }
            }

            $this->importedXml = @simplexml_load_string($imported_file);

            if ($this->importedXml === false) {
                throw new \Exception('Error parsing XML');
            }

            $transaction_identifier = $this->importedXml->xpath('//TransactionIdentifier');

            if ($transaction_identifier) {
                $transaction_identifier = reset($transaction_identifier);
            } else {
                throw new \Exception('TransactionIdentifier tag is missing');
            }

            $job = (Job::load((int) $transaction_identifier));

            if (empty($job)) {
                throw new \Exception('Cannot find matching job ID in the system');
            }
            return true;
        } catch (\Exception $e) {
            $this->importedXml = false;
            return $e->getMessage();
        }
    }

    /**
     * Job Item ids are stored in [lang][entity_type][entity_id].
     */
    protected function getMappedItemIds(JobInterface $job)
    {
        $job_items = $job->getItems();

        foreach ($job_items as $job_item) {
            $this->mappedItemsIDs[$job_item->get('cdt_target_language')->value][$job_item->get('item_type')->value][$job_item->get('item_id')->value] = $job_item->get('item_id')->value;
        }

    }

    /**
     * Import per target language.
     */
    protected function getImportedTargets(JobInterface $job, $target_language)
    {
        $this->getMappedItemIds($job);
        $importedTransUnits = [];

        $errors = 0;
        $warnings = 0;

        $fullImportedTransUnits = [];

        foreach ($this->importedXml->xpath('//Translation') as $translation) {

            if ($job->get('source_language')->value != $target_language) {
                $entity_field_list = [];
                foreach ($translation->StaticContent as $unit) {

                    $entity_id = (string) $unit['instanceId'];
                    $entity_type = (string) $unit['indexType'];
                    $entity_field = (string) $unit['resName'];
                    $res_type = (string) $unit['resType'];
                    $res_max_size = (int) $unit['resMaxSize'];

                    // Be sure we have a job item for this combinaison.
                    if (!empty($this->mappedItemsIDs[$target_language][$entity_type][$entity_id])) {

                        $job_items = $job->getItems(
                            [
                                'item_id' => $entity_id,
                                'cdt_target_language' => $target_language,
                            ]
                        );
                        $job_item = reset($job_items);

                        $job_item_id = $this->mappedItemsIDs[$target_language][$entity_type][$entity_id];
                        $key = $job_item_id . $entity_field;
                        $entity_field_list[$entity_field] = $key;

                        try {

                            if ((bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.html_tags_comparison')) {

                                $entity_field_clean = str_replace('[', '', $entity_field);
                                $entity_field_clean = str_replace(']', '', $entity_field_clean);

                                $tag_type_count = $job_item->getCdtSetting('tagsTypeCount', array());

                                if (isset($tag_type_count[$entity_field_clean])) {
                                    $source_tags = $tag_type_count[$entity_field_clean];
                                } else {
                                    $source_tags = [];
                                }

                                $target_tags = @TmgmtCdtFormatHelper::tagsTypeCount((string) $unit);

                                // Check tags quantity (but can import translation)
                                if (\count($source_tags) !== \count($target_tags)) {
                                    $warnings++;
                                    $job_item->addMessage(
                                        $this->t('The quantity of tags in the source and the target is different. Field: @field (@language)'),
                                        [
                                            '@field' => $entity_field,
                                            '@language' => $target_language,
                                        ], 'warning'
                                    );
                                }

                            }

                            $converted_content = TmgmtCdtFormatHelper::convertCdtToDrupal((string) $unit);

                            if (strpos($converted_content, 'tmgmt-cdt') !== false) {
                                throw new \Exception('The content has not been converted. Please check the translation. field: ' . $entity_field . ', language: ' . $target_language);
                            }

                            if ($res_type !== 'html') {
                                $converted_content = html_entity_decode($converted_content);
                                $preserve = '';
                                foreach (explode(';', TmgmtCdtHelper::getTranslatorSetting('conf_advanced.custom_keywords')) as $el) {
                                    if ($el[0] === '<' && substr($el, -1) === '>') {
                                        $preserve .= $el;
                                    }
                                }
                                // preserve drupal keywords like <none>, <front> .. etc
                                $converted_content = \strip_tags($converted_content, $preserve);
                            }
                            if ($res_max_size > 0) {
                                if (strlen($converted_content) > $res_max_size) {
                                    $warnings++;
                                    $job_item->addMessage(
                                        $this->t('The content is too long, it will probably be truncated. Field: @field (@language)'),
                                        [
                                            '@field' => $entity_field,
                                            '@language' => $target_language,
                                        ], 'warning'
                                    );
                                }
                            }

                            $importedTransUnits[$key]['#text'] = $converted_content;

                        } catch (\Exception $e) {
                            $importedTransUnits[$key]['#text'] = '';
                            $errors++;
                            $job_item->addMessage(
                                'ERROR @message (@language)',
                                [
                                    '@message' => $e->getMessage(),
                                    '@language' => $target_language,
                                ], 'error'
                            );
                        }
                    }

                }

                $data = \Drupal::service('tmgmt.data')->filterTranslatable($job_item->getData());
                // pour les champs non traduit, on utilise la version originale

                foreach ($data as $el => $value) {
                    if (!in_array('][' . $el, array_keys($entity_field_list))) {
                        $fullImportedTransUnits[$entity_id . '][' . $el] = ['#text' => $value['#text']];
                    } else {

                        $fullImportedTransUnits[$entity_id . '][' . $el] = $importedTransUnits[$entity_id . '][' . $el];
                    }
                }
            }

        }
        unset($importedTransUnits);

        if ($warnings > 0) {
            $this->messenger()->addWarning($this->t('The import has generated @warnings warnings (but translations are imported). Please consult the message list (at the bottom of the job page)', ['@warnings' => $warnings]), false);
        }

        if ($errors > 0) {
            $this->messenger()->addError($this->t('The import has generated  @errors errors (translations are not imported). Please consult the message list (at the bottom of the job page)', ['@errors' => $errors]), false);
        }

        return $fullImportedTransUnits;
    }

}
